"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = makeRequest;
var _abortController = _interopRequireDefault(require("abort-controller"));
var _formData = _interopRequireDefault(require("form-data"));
var _httpsProxyAgent = _interopRequireDefault(require("https-proxy-agent"));
var _asyncRetry = _interopRequireDefault(require("async-retry"));
var _nodeFetch = _interopRequireDefault(require("node-fetch"));
var _jose = require("jose");
var _package = require("../package.json");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
function prepareFormData(data) {
  if (!data) {
    return;
  }
  const form = new _formData.default();
  Object.keys(data).forEach(key => {
    const value = data[key];
    if (typeof value === 'object' && value.options) {
      // We have a file
      form.append(key, value.value, value.options);
    } else {
      form.append(key, value);
    }
  });
  return form;
}
async function makeRequest(requestAttributes, {
  apiKey,
  apiSecret,
  retryCount = 0,
  /* legacy */maxTries,
  timeout = 60000,
  retryMinTimeout,
  retryMaxTimeout
}, {
  HTTP_PROXY
} = process.env) {
  const {
    url,
    method = 'GET',
    formData,
    body: jsonBody
  } = requestAttributes;
  return (0, _asyncRetry.default)(async () => {
    const start = Date.now();

    // We must avoid reusing FormData instances when retrying requests
    // because they are consumed and cannot be reused.
    // More info: https://github.com/node-fetch/node-fetch/issues/1743
    const body = formData ? prepareFormData(formData) : jsonBody ? JSON.stringify(jsonBody) : undefined;
    const encodedSecret = new TextEncoder().encode(apiSecret);
    // https://github.com/panva/jose/blob/main/docs/classes/jwt_sign.SignJWT.md
    const signed = await new _jose.SignJWT({
      key: apiKey
    }).setProtectedHeader({
      alg: 'HS256',
      kid: apiKey
    }).sign(encodedSecret);
    const headers = {
      Authorization: `Bearer ${signed}`,
      'User-Agent': `happo.io@${_package.version}`
    };
    if (jsonBody) {
      headers['Content-Type'] = 'application/json';
    }
    const controller = new _abortController.default();
    const abortTimeout = setTimeout(() => {
      console.error(`Timing out request ${method} ${url} after ${timeout} ms.`);
      controller.abort();
    }, timeout);
    try {
      const response = await (0, _nodeFetch.default)(url, _objectSpread(_objectSpread({
        headers,
        compress: true,
        agent: HTTP_PROXY ? new _httpsProxyAgent.default(HTTP_PROXY) : undefined,
        signal: controller.signal
      }, requestAttributes), {}, {
        body
      }));
      if (!response.ok) {
        const e = new Error(`Request to ${method} ${url} failed: ${response.status} - ${await response.text()}`);
        e.statusCode = response.status;
        throw e;
      }
      const result = await response.json();
      return result;
    } catch (e) {
      if (e.type === 'aborted') {
        e.message = `Timeout when fetching ${url} using method ${method}`;
      }
      e.message = `${e.message} (took ${Date.now() - start} ms)`;
      throw e;
    } finally {
      clearTimeout(abortTimeout);
    }
  }, {
    retries: retryCount || maxTries,
    minTimeout: retryMinTimeout,
    maxTimeout: retryMaxTimeout,
    onRetry: e => {
      console.warn(`Failed ${method} ${url}. Retrying...`);
      console.warn(e);
    }
  });
}